/* .SH NAME 
 * wlb --- expose a locale-related bug in MagickWand
 * .SH SYNOPSIS
 * LANG=pl_PL.utf8 gdb wlb
 * .SH BUGS
 * The program crashes with segmentation violation when $LANG == pl_PL.utf8
 *  */

#define TRIGGER_CRASH /* undef to  inhibit the crash */

#include <stdlib.h>
#include <stdio.h>
#include <locale.h>
#include <assert.h>
#include <string.h>
#include <ImageMagick/wand/MagickWand.h>

struct stdnames 
{ char t_locale [07], t_write [06], t_spawn [05], t_empty [01]; } 
const sc_stdnames = { "locale", "write", "spawn" };

static void handle_Magick_error (MagickWand *p_wand)
/* Writes an error message to the standard error */
{ 
	ExceptionType a_sev; 
	register char *const a_descr = MagickGetException (p_wand, &a_sev); 
	if (0 > fprintf (stderr, "%s %s %lu %s", GetMagickModule (), a_descr)) 
{ perror (sc_stdnames. t_write); } MagickRelinquishMemory (a_descr); }
/* !handle_Magick_error X */

int main (int p_arg_count, char const *const p_args []) 
/* 
 * Fills the form with predefined strings.  
 * The form has four pages and the strings are placed on subsequent pages in order.
 */
{ register int a_code = +EXIT_FAILURE; 
		 if (setlocale (LC_ALL, "") == NULL) perror (sc_stdnames. t_locale);
MagickWandGenesis ();
{ register MagickWand *const a_wand = NewMagickWand (); 
if (MagickReadImage (a_wand, "canvas:white") == MagickTrue) /* ?READ_IMAGE_SUCCEEDED Y */  
{ 
	register DrawingWand *const a_draw = NewDrawingWand (); 
	register PixelWand *const a_px = NewPixelWand (); 	/* .H2 FORM DATA */
static unsigned char const sc_message [] = "c";
	PixelSetHSL (a_px, 0, 0, 0);
	#ifdef TRIGGER_CRASH
	DrawSetStrokeColor (a_draw, a_px); 
	#endif
	DrawAnnotation (a_draw, 0, 0, sc_message);
	if (MagickDrawImage (a_wand, a_draw) != MagickTrue) handle_Magick_error (a_wand); else a_code = +EXIT_SUCCESS;
DestroyPixelWand (a_px); DestroyDrawingWand (a_draw); } 
else /* ?READ_IMAGE_SUCCEEDED N */ handle_Magick_error (a_wand); /* ?READ_IMAGE_SUCCEEDED */ 
DestroyMagickWand (a_wand);
} 
MagickWandTerminus ();
if (fflush (stdout) < 0) 
{ perror (sc_stdnames. t_write); return +EXIT_FAILURE; } else return +a_code;
 }

